#pragma once
#include "opencv2/opencv.hpp"
#include <eigen3/Eigen/Eigen>
#include <sys/time.h>
#include "Feature.h"

namespace Utilities {
	//Intrinsic SIMD accelerated signal processing functions
	void   bilinearInterp_SIMD (cv::Mat& Source, int X, int Y, double centerX, double centerY, cv::Mat& Target);
	
	//Functions to compute measures of distance between matrices (Hand-optimized with SSE-AVX - single-threaded)
	double computeL1Diff_SIMD             (cv::Mat& A, cv::Mat& B);
	
	void UIntMatToFloatMat(cv::Mat& In, cv::Mat& Out);
	
	//Basic Math Functions
	int min_int(int x, int y);
	
	//Advanced Math Functions
	double coputeStandardDeviation(std::vector<double> &errors, std::vector<uchar> &pointUseFlags);
	double coputeStandardDeviation(std::vector<double> &errors);
	void homogeneousLinearLeastSquares(Eigen::MatrixXd &A, Eigen::VectorXd &v, Eigen::VectorXd &sigma);
	
	//Time Utilities
	bool   isFirstNewer       (struct timeval a,  struct timeval b);
	double timeDiff           (struct timeval a,  struct timeval b);
	void   moveForwardsInTime (struct timeval &a, double seconds);
	void   moveBackwardsInTime(struct timeval &a, double seconds);
	
	//Filesystem Utilities
	std::vector<std::string> glob(const std::string& pat);
	
	//Drawing Utilities
	void drawCenteredText(cv::Mat &Image, std::string& text, cv::Point location, double size, int thickness, int fontFace, cv::Scalar color=cv::Scalar::all(255));
	
	//User Interface Utilities
	int countDown(cv::VideoCapture cap, const char* windowName, int count, std::string line1, std::string line2);
	int waitForButtonPress(cv::VideoCapture cap, const char* windowName, int count, std::string line1, std::string line2);
	
	//Data Interchange Utilities
	int saveTrajectories(FeatureSet &features, std::string outputFilePath);
	int loadTrajectories(std::string inputFilePath, std::list<int> &birthFrameNumbers, std::list< std::list< cv::Vec2f > > &trajectories);
	
	int loadInitialPositions(std::string inputFilePath, std::list<cv::Vec2f> &initPositions);
};
